@@ -1,4 +1,4 @@
-__version__ = "1.5"
+__version__ = "1.7.0"
 __url__ = "https://gitlab.com/cest-group/boss"
 __description__ = "Bayesian optimization structure search"
 __authors__ = [
@@ -7,9 +7,13 @@ __authors__ = [
     "Arttu Tolvanen",
     "Ulpu Remes",
     "Nuutti Sten",
+    "Emma Lehto",
+    "Manuel Kuchelmeister",
+    "Tuomas Rossi",
+    "Mikael Granit"
     "Joakim Loefgren",
     "Milica Todorovic",
 ]
 __maintainer__ = "The BOSS developers team"
-__maintainer_email__ = "milica.todorovic@aalto.fi"
+__maintainer_email__ = "milica.todorovic@utu.fi"
 __license__ = "Apache License 2.0"
