@@ -1,6 +1,8 @@
 from itertools import repeat
 from multiprocessing import Pool
 
+import numpy as np
+
 
 class HMCwrapper:
     def __init__(self, sample):
@@ -45,5 +47,5 @@ class HMCwrapper:
         res = []
         for p in chunk:
             model.set_unfixed_params(p)
-            res.append(acqfunc(x, model, params))
+            res.append(acqfunc.evaluate_with_gradient(x))
         return res
