@@ -18,10 +18,10 @@ def split_rst_data(rst_data, dim):
     Returns
     -------
     acqs: np.ndarray
-        Array containing the interation number (starting from 1) and the correspoding
+        Array containing the iteration number (starting from 1) and the correspoding
         acquisitions found in the restart data.
     mod_par: np.ndarray
-        Array containing the interation number (starting from 1) and the correspoding
+        Array containing the iteration number (starting from 1) and the correspoding
         model parameters found in the restart data.
     """
     totalpts = rst_data.shape[0]
@@ -58,13 +58,30 @@ class RstManager:
         self.data = rst_data
         self.settings = settings
         self.x_dim = settings.dim
-        self.use_gradients = settings["ygrads"]
+        self.ygrad = settings["ygrad"]
         self.rstfile = settings["rstfile"]
 
+    @property
+    def X(self):
+        if self.data.shape[0] > 0:
+             return self.data[:, :self.x_dim]
+        else:
+            return None
+
+    @property
+    def Y(self):
+        if self.data.shape[0] > 0:
+             return self.data[:, self.x_dim: 1 + self.x_dim * (1 + self.ygrad)]
+        else:
+            return None
+
     def new_file(self):
         if os.path.isfile(self.rstfile):
             print("warning: overwriting file '" + self.rstfile + "'")
-        dump.dump_input_file(self.rstfile, self.settings, results_header=True)
+        keywords = {
+            k: v for k, v in self.settings.items() if k in self.settings.user_keys
+        }
+        dump.dump_input_file(self.rstfile, keywords, results_header=True)
 
     def get_x(self, i):
         """
@@ -88,7 +105,7 @@ class RstManager:
         """
         len_x = self.x_dim  # for convenience
         len_y = 1
-        if self.use_gradients:
+        if self.ygrad:
             len_y += len_x
         y_new = None
         yd_new = None
@@ -99,9 +116,9 @@ class RstManager:
             and sum(np.isnan(self.data[i, len_x : (len_x + len_y)])) == 0
         ):
             y_new = self.data[i, len_x]
-            if self.use_gradients:
+            if self.ygrad:
                 yd_new = self.data[i, (len_x + 1) : (len_x + len_y)]
-        if not self.use_gradients:
+        if not self.ygrad:
             return y_new
         else:
             return (y_new, yd_new)
@@ -113,7 +130,7 @@ class RstManager:
         """
         len_x = self.x_dim  # for convenience
         len_y = 1
-        if self.use_gradients:
+        if self.ygrad:
             len_y += len_x
         a = len_x + len_y
 
