@@ -18,11 +18,8 @@ def dump_input_file(new_file_path, keywords, results_header=False):
             fd.write("\nRESULTS:")
 
 
-def dump_model(settings, dest_file, bo, mod_params, xhat):
-    """
-    Outputs model slice (up to 2D) mean and variance in a grid to
-    models/it#.dat
-    """
+def build_query_points(settings, defaults):
+    """Build array of query points."""
     npts_per_dim = settings["pp_model_slice"][2]
     slice1 = settings["pp_model_slice"][0]
     slice2 = settings["pp_model_slice"][1]
@@ -46,9 +43,9 @@ def dump_model(settings, dest_file, bo, mod_params, xhat):
     # Full ND array of query points
     X = np.empty((npts, settings.dim))
 
-    # Fill ND query point array with default values (or xhat)
+    # Fill ND query point array with default values
     if settings["pp_var_defaults"] is None:
-        X[:] = xhat
+        X[:] = defaults
     else:
         X[:] = settings["pp_var_defaults"]
 
@@ -56,71 +53,69 @@ def dump_model(settings, dest_file, bo, mod_params, xhat):
     X[:, slice1] = x1
     if is_2d:
         X[:, slice2] = x2
+    return X
 
-    # Get data
-    mu, nu = bo.get_mu_and_nu(X)
-    model_data = np.hstack((X, mu, nu))
 
-    # Build header lines
-    header = "# Model output (x mu nu), grid of "
+def build_header_line(settings, name):
+    npts_per_dim = settings["pp_model_slice"][2]
+    slice1 = settings["pp_model_slice"][0]
+    slice2 = settings["pp_model_slice"][1]
+    is_2d = slice1 != slice2
+    header = f"# {name}, grid of "
     if is_2d:
         header += "%ix%i=" % (npts_per_dim, npts_per_dim)
+        npts = npts_per_dim**2
+    else:
+        npts = npts_per_dim
     header += "%i pts\n" % npts
+    return header
+
+
+def dump_model(settings, dest_file, model, mod_params, x_glmin):
+    """
+    Outputs model slice (up to 2D) mean and variance in a grid to
+    models/it#.dat
+    """
+    X = build_query_points(settings, x_glmin)
+    mu, var = model.predict(X)
+    nu = np.sqrt(var)
+    data = np.hstack((X, mu, nu))
+
+    header = build_header_line(settings, "Model output (x mu nu)")
     header += (
         "# Model parameter values "
         + "({}): ".format(' '.join(mod_params.keys()))
         + format(tuple(mod_params.values()))
+        + "\n"
     )
 
-    # Write
     with open(dest_file, 'w') as fd:
-        fd.write(header + '\n')
-        np.savetxt(fd, model_data, fmt='%18.8e')
+        fd.write(header)
+        np.savetxt(fd, data, fmt='%18.8e')
 
 
-def dump_acqfn(settings, dest_file, bo, acqfn, defs):
+def dump_acqfn(settings, dest_file, acqfn, defs):
     """
     Outputs acquisition function slice (up to 2D) in a grid to
     acqfns/it#.dat
     """
-    acqf_data = [[]] * (settings.dim + 1)  # coords + acqf
-    npts = settings["pp_model_slice"][2]
-    coords = np.array(
-        [
-            np.linspace(settings["bounds"][i, 0], settings["bounds"][i, 1], npts)
-            for i in settings["pp_model_slice"][:2]
-        ]
-    )
-    defaults = (
-        defs if settings["pp_var_defaults"] is None else settings["pp_var_defaults"]
-    )
-    if settings["pp_model_slice"][0] != settings["pp_model_slice"][1]:
-        # 2D slice
-        x1, x2 = np.meshgrid(coords[0], coords[1])
-        for i in range(npts):
-            for j in range(npts):
-                p = np.array([x1[i, j], x2[i, j]])
-                for d in range(settings.dim):
-                    if d not in settings["pp_model_slice"][:2]:
-                        p = np.insert(p, d, defaults[d])
-                af, daf = acqfn(p, bo.model, bo.acqfnpars)
-                p = np.insert(p, len(p), float(af))
-                acqf_data = np.insert(acqf_data, len(acqf_data[0]), p, axis=1)
-        titleLine = "# Acquisition function  (x" + " af), grid of %i pts" % (npts ** 2)
-    else:
-        # 1D slice
-        x1 = coords[0]
-        for i in range(npts):
-            p = np.array([x1[i]])
-            for d in range(settings.dim):
-                if d not in settings["pp_model_slice"][:2]:
-                    p = np.insert(p, d, defaults[d])
-            af, daf = acqfn(p, bo.model, bo.acqfnpars)
-            p = np.insert(p, len(p), float(af))
-            acqf_data = np.insert(acqf_data, len(acqf_data[0]), p, axis=1)
-        titleLine = "# Acquisition function (x" + " af), grid of %i pts" % (npts)
+    X = build_query_points(settings, defs)
 
-    ioutils.write_cols(dest_file, acqf_data, "    ", titleLine, "%18.8E")
+    if settings['costfn'] is None:
+        af = acqfn(X)
+        data = np.hstack((X, af))
+    else:
+        data = []
+        for x in X:
+            af = acqfn(x)
+            data.append(np.concatenate((x, af[0])))
+        data = np.array(data)
+
+    header = build_header_line(settings, "Acquisition function (x af)")
+
+    with open(dest_file, 'w') as fd:
+        fd.write(header)
+        np.savetxt(fd, data, fmt='%18.8e')
 
 
 def dump_truef(settings, dest_file, last_xhat):
@@ -149,7 +144,9 @@ def dump_truef(settings, dest_file, last_xhat):
                 for d in range(settings.dim):
                     if d not in settings["pp_model_slice"][:2]:
                         p = np.insert(p, d, defaults[d])
-                tf = settings.f(np.atleast_2d(p))
+
+                tf = settings.f(np.atleast_2d(p))[1][0].item()
+
                 os.chdir(settings.dir)
                 p = np.insert(p, len(p), float(tf))
                 truef_data = np.insert(truef_data, len(truef_data[0]), p, axis=1)
@@ -166,7 +163,7 @@ def dump_truef(settings, dest_file, last_xhat):
             for d in range(settings.dim):
                 if d not in settings["pp_model_slice"][:2]:
                     p = np.insert(p, d, defaults[d])
-            tf = settings.f(np.atleast_2d(p))
+            tf = settings.f(np.atleast_2d(p))[1][0].item()
             os.chdir(settings.dir)
             p = np.insert(p, len(p), float(tf))
             truef_data = np.insert(truef_data, len(truef_data[0]), p, axis=1)
