@@ -11,18 +11,12 @@ Keyword values are evaluated according to the ruleset:
     3                 -> int
     1.0               -> float
     foo bar           -> List[str]
-    3 10              -> np.ndarray(shape(1, 2), dtype=np.int64)
-    5.0 7.0           -> np.ndarray(shape(1, 2), dtype=np.float64)
-    1.0 2.0 ; 3.0 4.0 -> np.ndarray(shape=(2,2), dtype=np.float64)
+    3 10              -> np.ndarray(shape(1, 2), dtype=int)
+    5.0 7.0           -> np.ndarray(shape(1, 2), dtype=float)
+    1.0 2.0 ; 3.0 4.0 -> np.ndarray(shape=(2,2), dtype=float)
 """
-import time
-import importlib
-import inspect
-import os
-import sys
 from pathlib import Path
 import boss.keywords as bkw
-import itertools
 
 import numpy as np
 
@@ -43,7 +37,7 @@ def parse_input_file(file_path, skip=None):
         Skip parsing part of the input, can be either 'keywords' or 'results'.
 
     Returns
-    ----------
+    -------
     input_data: dict
         A dictionary containing the parsed input data. Can contain
         the following items:
@@ -104,37 +98,11 @@ def parse_input_file(file_path, skip=None):
     return input_data
 
 
-def parse_config():
-    """Parses a config file for BOSS keywords to use as new defaults.
-
-    The name of the config file must be .bossrc or bossrc. BOSS will search
-    for a config file under the following directories, in order:
-
-    $PWD > $HOME > $HOME/.config/boss
-
-    Only the first file encountered will be parsed.
-
-    Returns
-    ----------
-    keywords: dict
-        BOSS keywords parsed from the config file.
-    """
-    parents = [Path.cwd(), Path.home(), Path.home() / ".config/boss"]
-    stems = [".bossrc", "bossrc"]
-    keywords = {}
-    for parent, stem in itertools.product(parents, stems):
-        path = parent / stem
-        if path.is_file():
-            keywords = parse_input_file(path)["keywords"]
-            break
-    return keywords
-
-
 def parse_min_preds(settings, outfile):
     """
-    Extracts xhat, muhat and nuhat from all bo iterations in the output
-    file. Returns a numpy array where each row is [npts, xhat, muhat,
-    nuhat]. Works on output files with all verbosity levels.
+    Extracts x_glmin, mu_glmin and nu_glmin from all bo iterations in the output
+    file. Returns a numpy array where each row is [npts, x_glmin, mu_glmin,
+    nu_glmin]. Works on output files with all verbosity levels.
     """
     data = []
     npts = None
@@ -153,31 +121,25 @@ def parse_min_preds(settings, outfile):
     return np.array(data).astype(float)
 
 
-def parse_acqs(settings, outfile):
-    """
-    Extracts data acquisitions from all bo iterations in the output file.
-    Returns a numpy array where each element is [npts, x, y].
-    Note that there may be several acquisitions on the same iteration -
-    especially the initial points at the 0th iteration.
-    Works on output files with all verbosity levels.
-    """
-    acq = []
-    npts = 0
-
+def parse_acqs(settings, outfile, ygrad=False):
+    acqs = []
+    batch_ind_limits = [0]
+    n_values = 1 + settings.dim * (1 + ygrad)
     with open(outfile, "r") as f:
         line = f.readline()
         while len(line) > 0:
             if line.find("Data point added to dataset") != -1:
                 line = f.readline()
-                while len(line.split()) == settings.dim + 1 and line.find(".") != -1:
-                    line = line.split()
-                    npts += 1
-                    acq.append(np.concatenate([[npts], line[: settings.dim + 1]]))
+                ind_lim = 0
+                while len(line.split()) == n_values and line.find(".") != -1:
+                    ind_lim += 1
+                    acq = np.fromstring(line, sep=' ', dtype=float)
+                    acqs.append(acq)
                     line = f.readline()
+                batch_ind_limits.append(batch_ind_limits[-1] + ind_lim)
             else:
                 line = f.readline()
-
-    return np.array(acq).astype(float)
+    return np.array(acqs), batch_ind_limits
 
 
 def parse_best_acqs(settings, outfile):
@@ -205,9 +167,9 @@ def parse_best_acqs(settings, outfile):
 
 def parse_conv_measures(settings, outfile):
     """
-    Extracts dx_hat and dmu_hat from all bo iterations in the output
+    Extracts dx_glmin and dmu_glmin from all bo iterations in the output
     file. Returns a numpy array where each element is
-    [npts, dxhat, dmuhat].
+    [npts, dx_glmin, dmu_glmin].
     Works only on output files with verbosity levels of at least 1.
     """
     data = []
