@@ -23,7 +23,6 @@ code via the Settings object.
 """
 import copy
 import importlib
-import os
 import sys
 from inspect import getfile
 from pathlib import Path
@@ -33,16 +32,15 @@ import numpy as np
 
 categories = {}
 categories[(bool, 0)] = {
-    "ygrads": False,
+    "ygrad": False,
     "pp_models": False,
     "pp_acq_funcs": False,
-    "pp_truef_at_xhats": False,
+    "pp_truef_at_glmins": False,
     "initupdate": True,
     "ynorm": False,
 }
 categories[(int, 0)] = {
     "initpts": 5,
-    "verbosity": 1,
     "iterpts": None,
     "updatefreq": 1,
     "cores": 1,
@@ -55,7 +53,6 @@ categories[(int, 0)] = {
     "hmciters": 0,
     "minzacc": 15,
     "minfreq": 1,
-    "userfn_arrdim": 2,
     "seed": None,
     "parallel_optims": 0,
 }
@@ -72,7 +69,7 @@ categories[(float, 0)] = {
 }
 categories[(float, 1)] = {
     "yrange": [-10.0, 10.0],
-    "gm_tol": None,
+    "glmin_tol": None,
     "periods": None,
     "acqfnpars": np.array([]),
     "pp_var_defaults": None,
@@ -85,11 +82,15 @@ categories[(str, 0)] = {
     "rstfile": "boss.rst",
     "acqfn_name": "elcb",
     "inittype": "sobol",
+    "optimtype": "score",
     "thetaprior": "gamma",
     "userfn": None,
+    "costfn": None,
+    "costtype": 'divide',
 }
 categories[(str, 1)] = {
     "kernel": ["rbf"],
+    "var_types": None,
 }
 
 
@@ -104,7 +105,7 @@ def get_copied_categories():
 
 
 def find_category(key):
-    """Finds the category dict that contains a given key. """
+    """Finds the category dict that contains a given key."""
     for cat, cat_dict in categories.items():
         if key in cat_dict:
             return cat
@@ -140,7 +141,7 @@ def destringify(val_str, category):
         a string 'True' with category (bool, 1) will be evaluated to [True].
 
     Returns
-    ----------
+    -------
     Any
         The result of the string evaluation.
     """
