@@ -1,16 +1,31 @@
 import os
 import shutil
+import numpy as np
 
 from boss.mep.mep import MEP
 from boss.mep.space import Space
 import boss.io.dump as dump
 import boss.io.ioutils as ioutils
 import boss.io.parse as parse
-from boss.pp.pp_main import recreate_bo
-import boss.pp.plot as plot
+from boss.bo.kernel_factory import KernelFactory
+from boss.bo.model import Model
+from boss.bo.rstmanager import split_rst_data
+from boss.pp.plot import plot_model
+
+
+def recreate_model(settings, acqs, par):
+    """Loads given data and parameters to a given model object."""
+    X, Y = np.split(acqs, [-1], axis=1)
+    kernel = KernelFactory.construct_kernel(settings)
+    noise = settings["noise"]
+    ynorm = settings["ynorm"]
+    model = Model(X, Y, kernel, noise, ynorm)
+    model.set_unfixed_params(par)
+    return model
 
 
 class MEPMain:
+
     def __init__(self, settings, ipt_rstfile, minimafile, main_output):
         # create needed directories
         if os.path.isdir("mep"):
@@ -25,7 +40,7 @@ class MEPMain:
 
         # initialize and run
         mep = MEP(
-            self.bo,
+            self.model,
             self.space,
             self.minima,
             settings["mep_precision"],
@@ -43,13 +58,14 @@ class MEPMain:
         if self.minima.shape[1] == 2:
             self.plot2D(settings, mep)
 
-    def get_model(self, settings, ipt_rstfile, main_output):
-        acqs, mod_par = parse.parse_rst(settings, ipt_rstfile)
-        self.bo = recreate_bo(
+    def get_model(self, settings, rstfile, main_output):
+        input_data = parse.parse_input_file(rstfile)
+        rst_data = input_data['rst_data']
+        acqs, mod_par = split_rst_data(rst_data, settings.dim)
+        self.model = recreate_model(
             settings,
             acqs[:, 1:],
-            mod_par[mod_par.shape[0] - 1, 1:],
-            main_output,
+            mod_par[-1, 1:],
         )
 
     def get_minima(self, minimafile, main_output):
@@ -80,7 +96,7 @@ class MEPMain:
         minima = self.minima
         truef = None
 
-        plot.model(
+        plot_model(
             settings,
             "mep/minpaths.png",
             mdata,
