@@ -5,8 +5,9 @@ file based on given source array(s) of data.
 
 import numpy as np
 import matplotlib.pyplot as plt
+from boss.utils.arrays import shape_consistent_X
 
-plt.switch_backend("agg")
+# plt.switch_backend("agg")
 
 
 def plot_data_acquisitions(settings, dest_file, acqs, min_preds):
@@ -27,7 +28,7 @@ def plot_data_acquisitions(settings, dest_file, acqs, min_preds):
     plt.fill_between(itNo_mp, muhat - nuhat, muhat, color="lightgrey")
     plt.plot(itNo_mp, muhat + nuhat, "grey", linewidth=2)
     plt.plot(itNo_mp, muhat - nuhat, "grey", linewidth=2)
-    plt.plot(itNo_mp, muhat, "black", linewidth=5, label=r"$\mu(\hat{x})$")
+    plt.plot(itNo_mp, muhat, "black", linewidth=5, label=r"$\mu_{\mathrm{glmin}}$")
     plt.scatter(
         itNo_acqs,
         y,
@@ -35,12 +36,12 @@ def plot_data_acquisitions(settings, dest_file, acqs, min_preds):
         linewidth=4,
         facecolors="none",
         edgecolors="black",
-        label="$acq_y$",
+        label=r"$acq_y$",
     )
 
     plt.xlim(min(itNo_mp), max(itNo_mp))
     plt.xticks(itNo_mp[:: max(1, round(len(itNo_mp) / 10))])
-    plt.ylabel("$y$ with $\mu(\hat{x})$", size=20)
+    plt.ylabel(r"$y$ with $\mu_{\mathrm{glmin}}$", size=20)
     plt.gca().tick_params(labelsize=18)
     plt.gca().ticklabel_format(useOffset=False)
 
@@ -63,7 +64,7 @@ def plot_data_acquisitions(settings, dest_file, acqs, min_preds):
             xhat[:, i],
             color=colors[i % len(colors)],
             linewidth=5,
-            label="$\hat{x}_%i$" % (i + 1),
+            label=r"$x_{\mathrm{glmin}}^%i$" % (i + 1),
         )
         plt.scatter(
             itNo_acqs,
@@ -72,13 +73,13 @@ def plot_data_acquisitions(settings, dest_file, acqs, min_preds):
             linewidth=4,
             facecolors="none",
             edgecolors=colors[i % len(colors)],
-            label="$acq_{x%i}$" % (i + 1),
+            label=r"$acq_{x%i}$" % (i + 1),
         )
 
     plt.xlim(min(itNo_mp), max(itNo_mp))
     plt.xticks(itNo_mp[:: max(1, round(len(itNo_mp) / 10))])
-    plt.xlabel("iteration", size=20)
-    plt.ylabel("$x_i$ with $\hat{x}_i$", size=20)
+    plt.xlabel("Iteration", size=20)
+    plt.ylabel(r"$x_i$ with $x_{\mathrm{glmin}}^i$", size=20)
     plt.gca().tick_params(labelsize=18)
     plt.gca().ticklabel_format(useOffset=False)
 
@@ -100,7 +101,7 @@ def plot_conv_measures(settings, dest_file, conv_meas, legends=True):
     plt.subplot(211)
     plt.plot(itNo, dxhat, "k", linewidth=5)
     # plt.xlabel('iteration', size=20)
-    plt.ylabel(r"$\Delta \hat{x}$", size=24)
+    plt.ylabel(r"$\Delta x_{\mathrm{glmin}}$", size=24)
     plt.xticks(itNo[:: max(1, round(len(itNo) / 10))])
     plt.grid(True)
     plt.gca().tick_params(labelsize=18)
@@ -110,8 +111,8 @@ def plot_conv_measures(settings, dest_file, conv_meas, legends=True):
 
     plt.subplot(212)
     plt.plot(itNo, dmuhat, "k", linewidth=5)
-    plt.xlabel("iteration", size=20)
-    plt.ylabel(r"$abs(\Delta \mu(\hat{x}))/\Delta y$", size=24)
+    plt.xlabel("Iteration", size=20)
+    plt.ylabel(r"$abs(\Delta \mu_{\mathrm{glmin}}/\Delta y$", size=24)
     plt.xticks(itNo[:: max(1, round(len(itNo) / 10))])
     plt.grid(True)
     plt.gca().tick_params(labelsize=18)
@@ -170,7 +171,7 @@ def plot_hyperparameters(settings, dest_file, hypers, legends=True):
             markersize=12,
             label=r"$\ell_ %i$" % (i + 1),
         )
-    plt.xlabel("iteration", size=20)
+    plt.xlabel("Iteration", size=20)
     plt.ylabel("Lengthscale", size=20)
     plt.xticks(itNo[:: max(1, round(len(itNo) / 10))])
     plt.yscale("log")
@@ -204,7 +205,7 @@ def plot_truef_hat(settings, dest_file, truef_hats, legends=True):
     plt.plot(itNo, abs(truehat - last), "k", linewidth=5)
     plt.plot(itNo, abs(best - last), "k", linestyle="dashed", linewidth=3, label="best")
     plt.grid(True)
-    plt.ylabel(r"$f(\hat{x})-f(\hat{x}_{last})$", size=24)
+    plt.ylabel(r"$f(x_{\mathrm{glmin}})-f(x_{\mathrm{glmin,last}})$", size=24)
     plt.xticks(itNo[:: max(1, round(len(itNo) / 10))])
     plt.gca().tick_params(labelsize=18)
     plt.gca().ticklabel_format(useOffset=False)
@@ -215,8 +216,8 @@ def plot_truef_hat(settings, dest_file, truef_hats, legends=True):
 
     plt.subplot(212)
     plt.plot(itNo, abs(tfhat_muhat), "k", linewidth=5)
-    plt.xlabel("iteration", size=20)
-    plt.ylabel(r"$f(\hat{x})-\mu(\hat{x})$", size=24)
+    plt.xlabel("Iteration", size=20)
+    plt.ylabel(r"$f(x_{\mathrm{glmin}})-\mu_{\mathrm{glmin}}$", size=24)
     plt.xticks(itNo[:: max(1, round(len(itNo) / 10))])
     plt.grid(True)
     plt.gca().tick_params(labelsize=18)
@@ -250,6 +251,9 @@ def plot_model(
     """
     Plots a (max 2D) slice of the model.
     """
+    if xnext is not None:
+        xnext = shape_consistent_X(xnext, settings.dim)
+
     coords = model_data[:, : settings.dim]
     mu, nu = model_data[:, -2], model_data[:, -1]
     slice_dim = (
@@ -257,58 +261,67 @@ def plot_model(
     )
     if axis_labels is None:
         axis_labels = [
-            "$x_%i$" % (settings["pp_model_slice"][0] + 1),
-            "$x_%i$" % (settings["pp_model_slice"][1] + 1),
+            r"$x_%i$" % (settings["pp_model_slice"][0] + 1),
+            r"$x_%i$" % (settings["pp_model_slice"][1] + 1),
         ]
 
     if slice_dim == 1:
         x1 = coords[:, settings["pp_model_slice"][0]]
+
         if truef is not None:
             crds = truef[:, : settings.dim]
             tf = truef[:, -1]
             tfx1 = crds[:, settings["pp_model_slice"][0]]
             plt.plot(tfx1, tf, "k", linewidth=5, label="f(x)")
+
         if incl_uncert:
             plt.fill_between(x1, mu + nu, mu, color="lightgrey")
             plt.fill_between(x1, mu - nu, mu, color="lightgrey")
-            plt.plot(x1, mu + nu, "grey", linewidth=3, label="$\\nu(x)$")
+            plt.plot(x1, mu + nu, "grey", linewidth=3, label=r"$\nu(x)$")
             plt.plot(x1, mu - nu, "grey", linewidth=3)
-        plt.plot(x1, mu, "b", linewidth=5, label="$\\mu(x)$")
+
+        plt.plot(x1, mu, "b", linewidth=5, label=r"$\mu(x)$")
 
         if xhat is not None:
             plt.axvline(
                 xhat[settings["pp_model_slice"][0]],
                 color="red",
                 linewidth=5,
-                label="$\hat{x}$",
+                label=r"$x_{\mathrm{glmin}}$",
                 zorder=19,
             )
 
-        if acqs is not None:
-            x1 = acqs[:, settings["pp_model_slice"][0]]
-            y = acqs[:, -1]
-            plt.scatter(
-                x1,
-                y,
-                s=200,
-                linewidth=6,
-                facecolors="none",
-                edgecolors="brown",
-                label="acqs",
-                zorder=18,
-            )
+        if settings.dim == 1:
+            # plot acqs
+            if acqs is not None:
+                x1 = acqs[:, settings["pp_model_slice"][0]]
+                y = acqs[:, settings.dim]
+                plt.scatter(
+                    x1,
+                    y,
+                    s=200,
+                    linewidth=6,
+                    facecolors="none",
+                    edgecolors="brown",
+                    label="acqs",
+                    zorder=18,
+                )
 
-        if xnext is not None:
-            plt.axvline(
-                xnext[settings["pp_model_slice"][0]],
-                color="green",
-                linewidth=5,
-                label="$x_{next}$",
-                linestyle="dashed",
-                zorder=20,
-            )
+            # plot xnext
+            if xnext is not None:
+                label = r"$x_{\mathrm{next}}$"
+                for xn in xnext:
+                    plt.axvline(
+                        xn[settings["pp_model_slice"][0]],
+                        color="green",
+                        linewidth=5,
+                        label=label,
+                        linestyle="dashed",
+                        zorder=20,
+                    )
+                    label = None
 
-        if minima is not None:
+        if settings['pp_var_defaults'] is None and minima is not None:
             x1 = minima[:, settings["pp_model_slice"][0]]
             y = minima[:, -2]
             plt.scatter(
@@ -329,7 +342,7 @@ def plot_model(
         yd = max(mu) - min(mu)
         plt.ylim(min(mu) - 0.1 * yd, max(mu) + 0.1 * yd)
         plt.xlabel(axis_labels[0], size=24)
-        plt.ylabel("$y$", size=24)
+        plt.ylabel(r"$y$", size=24)
         if legends:
             lgd = plt.legend(
                 bbox_to_anchor=(0.0, 1.02, 1.0, 0.102),
@@ -401,6 +414,8 @@ def plot_acq_func(
     Plots a (max 2D) slice of the acquisition function. Can include also
     acquisitions, xhat and xnext if those are given.
     """
+    if xnext is not None:
+        xnext = shape_consistent_X(xnext, settings.dim)
     coords = acqf_data[:, : settings.dim]
     af = acqf_data[:, -1]
     slice_dim = (
@@ -408,8 +423,8 @@ def plot_acq_func(
     )
     if axis_labels is None:
         axis_labels = [
-            "$x_%i$" % (settings["pp_model_slice"][0] + 1),
-            "$x_%i$" % (settings["pp_model_slice"][1] + 1),
+            r"$x_%i$" % (settings["pp_model_slice"][0] + 1),
+            r"$x_%i$" % (settings["pp_model_slice"][1] + 1),
         ]
 
     if slice_dim == 1:
@@ -434,17 +449,18 @@ def plot_acq_func(
                 xhat[settings["pp_model_slice"][0]],
                 color="red",
                 linewidth=5,
-                label="$\hat{x}$",
+                label=r"$x_{\mathrm{glmin}}$",
             )
 
         if xnext is not None:
-            plt.axvline(
-                xnext[settings["pp_model_slice"][0]],
-                color="green",
-                linewidth=5,
-                label="$x_{next}$",
-                linestyle="dashed",
-            )
+            for xn in xnext:
+                plt.axvline(
+                    xn[settings["pp_model_slice"][0]],
+                    color="green",
+                    linewidth=5,
+                    label=r"$x_{\mathrm{next}}$",
+                    linestyle="dashed",
+                )
 
         plt.xlim(
             min(coords[:, settings["pp_model_slice"][0]]),
@@ -453,7 +469,7 @@ def plot_acq_func(
         yd = max(af) - min(af)
         plt.ylim(min(af) - 0.1 * yd, max(af) + 0.1 * yd)
         plt.xlabel(axis_labels[0], size=24)
-        plt.ylabel("$acqfn(x)$", size=24)
+        plt.ylabel(r"$acqfn(x)$", size=24)
         if legends:
             lgd = plt.legend(
                 bbox_to_anchor=(0.0, 1.02, 1.0, 0.102),
@@ -474,7 +490,7 @@ def plot_acq_func(
 
     elif slice_dim == 2:
         x1, x2, af = _get_2d_arrays(coords, settings, af)
-        _plot_2d_contours(x1, x2, af, label='$af(x)$')
+        _plot_2d_contours(x1, x2, af, label=r'$af(x)$')
 
         lo = False
         lo |= _plot_2d_acqs(acqs, settings)
@@ -513,8 +529,8 @@ def plot_truef(settings, dest_file, truef_data, axis_labels=None):
     )
     if axis_labels is None:
         axis_labels = [
-            "$x_%i$" % (settings["pp_model_slice"][0] + 1),
-            "$x_%i$" % (settings["pp_model_slice"][1] + 1),
+            r"$x_%i$" % (settings["pp_model_slice"][0] + 1),
+            r"$x_%i$" % (settings["pp_model_slice"][1] + 1),
         ]
 
     if slice_dim == 1:
@@ -522,7 +538,7 @@ def plot_truef(settings, dest_file, truef_data, axis_labels=None):
         plt.plot(x1, tf, "k", linewidth=5)
         plt.xlim(min(x1), max(x1))
         plt.xlabel(axis_labels[0], size=24)
-        plt.ylabel("$f(x)$", size=24)
+        plt.ylabel(r"$f(x)$", size=24)
         plt.title("True function", size=20)
         plt.gcf().set_size_inches(10, 8)
         plt.gca().tick_params(labelsize=18)
@@ -534,7 +550,7 @@ def plot_truef(settings, dest_file, truef_data, axis_labels=None):
     elif slice_dim == 2:
         x1, x2, tf = _get_2d_arrays(coords, settings, tf,
                                     npts=settings["pp_truef_npts"])
-        _plot_2d_contours(x1, x2, tf, label='$f(x)$')
+        _plot_2d_contours(x1, x2, tf, label=r'$f(x)$')
         _plot_2d_labels_settings(axis_labels)
         plt.title("True function", size=20)
         plt.tight_layout()
@@ -575,7 +591,7 @@ def _plot_2d_xhat(xhat, settings):
         xhat[settings["pp_model_slice"][1]],
         "r*",
         markersize=26,
-        label="$\hat{x}$",
+        label=r"$x_{\mathrm{glmin}}$",
         zorder=10,
     )
     return True
@@ -614,11 +630,11 @@ def _plot_2d_xnext(xnext, settings):
     if xnext is None:
         return False
     plt.plot(
-        xnext[settings["pp_model_slice"][0]],
-        xnext[settings["pp_model_slice"][1]],
+        xnext[:, settings["pp_model_slice"][0]],
+        xnext[:, settings["pp_model_slice"][1]],
         "b^",
         markersize=26,
-        label="$x_{next}$",
+        label=r"$x_{next}$",
         zorder=10,
     )
     return True
