@@ -1,69 +1,24 @@
-import importlib
-import inspect
-import os
-import sys
-from pathlib import Path
-import copy
+from collections import UserDict
 
+import os
+import copy
 import numpy as np
 
 import boss.keywords as bkw
 import boss.io.parse as parse
 import boss.io.dump as dump
-from boss.bo.acq.ei import ei
-from boss.bo.acq.elcb import elcb
-from boss.bo.acq.exploit import exploit
-from boss.bo.acq.explore import explore
-from boss.bo.acq.lcb import lcb
-from boss.io.main_output import MainOutput
-from boss.utils.distributions import Distributions
+from boss.bo.acq.ei import EI
+from boss.bo.acq.elcb import ELCB
+from boss.bo.acq.exploit import Exploit
+from boss.bo.acq.explore import Explore
+from boss.bo.acq.lcb import LCB
+from boss.bo.acq.cost import AdditiveCost, DivisiveCost
+from boss.utils.distributions import gammaparams
+from boss.bo.userfunc import UserFunc
+from boss.bo.acq.cost import CostFunc
 
 
-def standardize_signature(f, xdim):
-    """Standardizes the signature of the user-function.
-
-    Internally, 2d np.ndarray are passed to the user function,
-    thus user-functions with different signatures must first
-    be converted to this format.
-
-    Parameters
-    ----------
-    f: Callable
-        User-function that takes an arbitary number
-        of type-identical arguments,
-    xdim: int
-        The dimension, as returned by np.ndim, of the individual arguments to f.
-
-    Returns
-    -------
-    Callable
-        A wrapped (if necessary) version of f that takes a single
-        1d np.ndarray as input.
-    """
-    sig = inspect.signature(f)
-    nargs = len(sig.parameters)
-    if xdim == 0:
-        def fnew(x):
-            return f(*x[0])
-    elif xdim == 1:
-        if nargs == 1:
-            def fnew(x):
-                return f(x[0])
-        else:
-            def fnew(x):
-                return f(*np.hsplit(x[0], nargs))
-    elif xdim == 2:
-        if nargs == 1:
-            fnew = f
-        else:
-            def fnew(x):
-                return f(*np.hsplit(x, nargs))
-    else:
-        raise ValueError
-    return fnew
-
-
-class Settings(dict):
+class Settings(UserDict):
     """Reads, interprets and defines the code internal settings based on input.
     """
 
@@ -75,14 +30,13 @@ class Settings(dict):
 
         self.dir = os.getcwd()
         self.acqfn = None
+        self.costfn = None
+        self.user_keys = keywords.keys()
 
         # Before any other keywords are set, we assign values to all keywords
         # with independent defaults.
         self.set_independent_defaults(only_missing=False)
 
-        # Update with keywords from config files
-        self.update(parse.parse_config())
-
         # Update with BOSS keywords passed to init
         self.update(keywords)
         if keywords.get("bounds") is None:
@@ -90,21 +44,34 @@ class Settings(dict):
                 "Keyword 'bounds' has to be defined by the user"
             )
 
+        # correct keyword types
+        self.correct()
+
         # Handle the user function: if a function is passed directly we take that,
         # otherwise fall back to the function specified by the userfn keyword.
+        self.f = None
         if f is None:
             if self["userfn"] is not None:
-                f_user = bkw.func_from_keyword(self["userfn"])
-                f = standardize_signature(f_user, self["userfn_arrdim"])
-            self.f = f
+                f = bkw.func_from_keyword(self["userfn"])
+                self.f = UserFunc(f, self.dim)
         else:
             self["userfn"] = bkw.func_to_keyword(f)
-            self.f = standardize_signature(f, self["userfn_arrdim"])
+            self.f = UserFunc(f, self.dim)
+
+        # Handle cost function: take either directly passed function or
+        # function specified by costfn keyword
+        if isinstance(self["costfn"], str):
+            costfn = bkw.func_from_keyword(self["costfn"])
+            self.costfn = CostFunc(costfn, self.dim)
+            
+        else:
+            if callable(self["costfn"]):
+                self.costfn = CostFunc(self["costfn"], self.dim)
+                self["costfn"] = bkw.func_to_keyword(self.costfn.func)
 
         # Set default values for dependent keywords if they
         # have not yet been specified.
         self.set_acqfn(self["acqfn_name"])
-        self.correct()
         self.set_dependent_defaults(only_missing=True)
 
         # Set RNG seed if specified.
@@ -117,7 +84,7 @@ class Settings(dict):
         """Factory method for Constructing a Settings object from a boss input file.
 
         Parameters
-        -----------
+        ----------
         file_path: str, Path
             Path to the input file.
 
@@ -145,25 +112,22 @@ class Settings(dict):
                     if self.get(key) is None:
                         self[key] = val
 
+
     def set_dependent_defaults(self, only_missing=True):
-        """Sets default values for keywords that depend on other keywords.
-        """
+        """Sets default values for keywords that depend on other keywords. """
         should_update = lambda key: self.get(key) is None or only_missing is False
 
         if should_update("periods"):
             self["periods"] = self["bounds"][:, 1] - self["bounds"][:, 0]
         if should_update("iterpts"):
             self["iterpts"] = int(15 * self.dim ** 1.5)
-        if should_update("pp_iters"):
-            self["pp_iters"] = list(range(self["iterpts"] + 1))
-            self["pp_iters"] = np.arange(0, self['iterpts'] + 1)
         if should_update("min_dist_acqs"):
             self["min_dist_acqs"] = 0.01 * min(self["periods"])
 
         # Default initial hyperparameters.
         if should_update("thetainit"):
             if self["ynorm"]:
-                diff = 1.0 # when observations are range normalised
+                diff = 1.0 # when observations are range-normalized
             else:
                 diff = self["yrange"][1] - self["yrange"][0]
             self["thetainit"] = [0.5 * diff]  # sig
@@ -212,7 +176,7 @@ class Settings(dict):
                             shape = 3.3678
                             rate = 9.0204
                         else:  # nonpbc
-                            shape, rate = Distributions.gammaparams(
+                            shape, rate = gammaparams(
                                 self["periods"][i] / 20, self["periods"][i] / 2
                             )
                         self["thetapriorpar"].append([shape, rate])
@@ -223,7 +187,7 @@ class Settings(dict):
                     )
 
         # Model slice and number of points.
-        if should_update("pp_model_slice"):
+        if should_update('pp_model_slice'):
             if self.dim == 1:
                 self["pp_model_slice"] = np.array([1, 1, 50])
             else:
@@ -249,22 +213,43 @@ class Settings(dict):
         raise AttributeError("Cannot modify read-only attribute: dim")
 
     def set_acqfn(self, name):
-        if name == "elcb":
-            self.acqfn = elcb
-        elif name == "lcb":
-            self.acqfn = lcb
-            if len(self["acqfnpars"]) < 1:
-                self["acqfnpars"] = np.array([2.0])  # explr_weight
-        elif name == "explore":
-            self.acqfn = explore
-        elif name == "exploit":
-            self.acqfn = exploit
-        elif name == "ei":
-            self.acqfn = ei
-        else:
-            raise TypeError(f"ERROR: Unknown acquisition function selected: {name}")
+        """Initializes the acquisition function based on settings.
 
-    def dump(self, file_path):
+        Wraps the acquisition function by a cost function, if
+        a cost function is set by the user.
+        """
+        self["acqfnpars"] = np.atleast_1d(self["acqfnpars"])
+        if name != 'explore':
+            self.expfn = Explore()
+        if name == 'ei':
+            if len(self["acqfnpars"]) < 1:
+                self["acqfnpars"] = np.array([0.0])
+            self.acqfn = EI(self["acqfnpars"])
+        elif name == 'elcb':
+            self.acqfn = ELCB()
+        elif name == 'exploit':
+            self.acqfn = Exploit()
+        elif name == 'explore':
+            self.acqfn = Explore()
+            self.expfn = self.acqfn
+        elif name == 'lcb':
+            if len(self["acqfnpars"]) < 1:
+                self["acqfnpars"] = np.array([2.0])
+            self.acqfn = LCB(self["acqfnpars"])
+        else:
+            raise TypeError(f"Unknown acquisition function selected: {name}")
+
+        if callable(self.costfn):
+            if self["costtype"] == "add":
+                self.acqfn = AdditiveCost(self.acqfn, self.costfn)
+                self.expfn = AdditiveCost(self.expfn, self.costfn)
+            elif self["costtype"] == "divide":
+                self.acqfn = DivisiveCost(self.acqfn, self.costfn)
+                self.expfn = DivisiveCost(self.expfn, self.costfn)
+            else:
+                raise ValueError(f'Unknown costtype {self["costtype"]}.')
+
+    def dump(self, file_path, only_user_keywords=True):
         """Writes the current settings to a boss input file.
 
         Parameters
@@ -272,33 +257,29 @@ class Settings(dict):
         fname : Union[str, path]
             Path to the destination file.
         """
-        dump.dump_input_file(file_path, self)
+        if only_user_keywords:
+            keywords = {k: v for k, v in self.items() if k in self.user_keys}
+        else:
+            keywords = self
+        dump.dump_input_file(file_path, keywords)
 
     def correct(self):
         """Corrects the type and value of certain keywords.
 
-        The user is allowed to be lazy in defining certain keywords,
-        e.g., by providing lists instead of np.arrays, and we correct
-        for this laziness here.
+        The user is afforded some laziness defining certain keywords,
+        e.g., by providing lists instead of np.arrays.
         """
-        if self["pp_acq_funcs"] and self["verbosity"] < 2:
-            self["verbosity"] = 2  # to have xnexts printed
+        # Make sure we're not using deprecated keywords
+        self.deprecation_notice()
 
         # Make sure int and float arrays are np and not python sequences.
-        # and have the correct dtype
         for key, val in self.items():
             cat = bkw.find_category(key)
             cat_type, cat_dim = cat[0], cat[1]
+            if cat_dim > 0 and cat_type in [int, float] and val is not None:
+                self[key] = np.asarray(val, dtype=cat_type)
 
-            if cat_dim > 0 and cat_type != str and val is not None:
-                if cat_type == int:
-                    self[key] = np.asarray(val, dtype=np.int64)
-                elif cat_type == float:
-                    self[key] = np.asarray(val, dtype=np.float64)
-
-        bounds = self['bounds']
-        if bounds.ndim == 1:
-            self['bounds'] = np.atleast_2d(bounds)
+        self['bounds'] = np.atleast_2d(self['bounds'])
 
         kernel = self["kernel"]
         if isinstance(kernel, str):
@@ -307,8 +288,25 @@ class Settings(dict):
         if len(self["kernel"]) == 1:
             self["kernel"] *= self.dim
 
-        # Exception to the above rule: gm_tol is a mixed list [float, int]
-        # TODO: break gm_tol into two keywords in a future release
-        gm_tol = self["gm_tol"]
-        if gm_tol is not None:
-            self["gm_tol"] = [float(gm_tol[0]), int(gm_tol[1])]
+        # Exception to the above rule: glmin_tol is a mixed list [float, int]
+        # TODO: break glmin_tol into two keywords in a future release
+        glmin_tol = self["glmin_tol"]
+        if glmin_tol is not None:
+            self["glmin_tol"] = [float(glmin_tol[0]), int(glmin_tol[1])]
+
+    def deprecation_notice(self):
+        deprecated = {
+            'pp_truef_at_xhats': 'pp_truef_at_glmins',
+            'gm_tol': 'glmin_tol',
+            'verbosity': None
+        }
+        msg = ''
+        for key, val in deprecated.items():
+            if key in self:
+                if val is None:
+                    msg += f'Keyword {key} has been deprecated and should be removed\n'
+                else:
+                    msg += f'Keyword {key} has been renamed, use {val} instead\n'
+
+        if len(msg) > 0:
+            raise RuntimeError(msg)
