@@ -5,7 +5,6 @@ warnings.filterwarnings("ignore")  # ignore warnings
 import numpy as np
 import copy
 from operator import itemgetter
-import numpy as np
 import scipy.optimize
 from scipy.spatial.distance import euclidean
 
@@ -204,7 +203,7 @@ class Minimization:
 
     @staticmethod
     def minimize_from_random(
-        func, bounds, num_pts, shift, args=(), lowest_min_only=True
+        func, bounds, num_pts, args=(), lowest_min_only=True
     ):
         """
         Tries to find global minimum of func by starting minimizers from
@@ -230,7 +229,7 @@ class Minimization:
         minima = Minimization._minimize(arguments)
 
         if len(minima) == 0:
-            err_min = [list(points[-1]), float(func(points[-1], args[0], args[1])[0])]
+            err_min = [list(points[-1]), float(func(points[-1])[0])]
             minima.append(err_min)  # later a better solution here?
 
         if lowest_min_only:
@@ -242,6 +241,44 @@ class Minimization:
         else:
             return minima
 
+    @staticmethod
+    def minimize_using_score(
+        func, bounds, num_rnd_samples=10000, num_anchor=1, acqs=None, args=(),
+            lowest_min_only=True):
+        """Searches the global minimum of func by starting minimizers from
+        num_anchor points. The points are chosen according to a score
+        criteria, here the function itself is used to obtain the scores.
+        """
+        uniform_samples = np.random.uniform(
+            low=bounds[:, 0], high=bounds[:, 1],
+            size=(num_rnd_samples, len(bounds)))
+        scores = func(uniform_samples)
+        if isinstance(scores, tuple):
+            scores = scores[0]   # Ignore gradient
+        sorted_idxs = np.argsort(scores.squeeze())
+        starting_points = uniform_samples[sorted_idxs[:num_anchor]]
+        if acqs is not None:
+            # Add point with lowest func value to starting_points
+            starting_points = np.vstack((starting_points, acqs))
+        min_args = [(func, point, bounds, args) for point in starting_points]
+        minima = Minimization._minimize(min_args)
+
+        if len(minima) == 0:
+            lowest_value = func(uniform_samples[sorted_idxs[0]])
+            if isinstance(lowest_value, tuple):
+                lowest_value = lowest_value[0].squeeze()
+            minima.append([uniform_samples[sorted_idxs[0]].tolist(),
+                           lowest_value])
+        if lowest_min_only:
+            lowest_min = minima[0]
+            for minimum in minima:
+                if minimum[1] < lowest_min[1]:
+                    lowest_min = minimum
+            return lowest_min
+        else:
+            return minima
+
+
     @staticmethod
     def _remove_duplicates(original, min_distance):
         """
